unit DemoUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBTables, DB, qiField, qiQuery, qiSelect, qiWdw, TablePrn, MemoDlg,
  qiFrBldr, qiLookup, qiOrder, qiGroup, qiFilter, qiFormat, qiArchiv,
  StdCtrls, qiModify, qiDefine, qiUpdate, UpdtUnit, OptsUnit, qiTables,
  qiAutGrp, objdebv;

type
  TDemoForm = class(TForm)
    Customers: TTable;
    Employees: TTable;
    Vendors: TTable;
    Orders: TTable;
    Items: TTable;
    Parts: TTable;
    CustomersCustNo: TFloatField;
    CustomersCompany: TStringField;
    CustomersAddr1: TStringField;
    CustomersCity: TStringField;
    CustomersTaxRate: TFloatField;
    EmployeesEmpNo: TIntegerField;
    EmployeesLastName: TStringField;
    VendorsVendorNo: TFloatField;
    VendorsVendorName: TStringField;
    VendorsCity: TStringField;
    VendorsPreferred: TBooleanField;
    OrdersCustNo: TFloatField;
    OrdersSaleDate: TDateTimeField;
    OrdersEmpNo: TIntegerField;
    OrdersPaymentMethod: TStringField;
    OrdersTaxRate: TFloatField;
    OrdersAmountPaid: TCurrencyField;
    ItemsOrderNo: TFloatField;
    ItemsItemNo: TFloatField;
    ItemsPartNo: TFloatField;
    ItemsQty: TIntegerField;
    ItemsDiscount: TFloatField;
    PartsPartNo: TFloatField;
    PartsVendorNo: TFloatField;
    PartsDescription: TStringField;
    PartsListPrice: TCurrencyField;
    QueryWdw: TQIQueryWindow;
    SelectDlg: TQISelectDialog;
    OrdersOrderNo: TFloatField;
    Label1: TLabel;
    Label2: TLabel;
    QueryBtn: TButton;
    Archiver: TQIArchiver;
    Formatter: TQIFormatter;
    FilterDlg: TQIFilterDialog;
    OrderDlg: TQIOrderDialog;
    LookupDlg: TQILookupDialog;
    FilterBldr: TQIFilterBuilder;
    EditDlg: TMemoDialog;
    CloseBtn: TButton;
    Label3: TLabel;
    UpdateBtn: TButton;
    DefineDlg: TQIDefineDialog;
    ModifyDlg: TQIModifyDialog;
    GroupDlg: TQIAutoGroupDialog;
    TablePrn: TTablePrinter;
    FieldInfo: TQIFieldInfo;
    QueryInfo: TQIQueryInfo;
    UpdateInfo: TQIUpdateInfo;
    procedure QueryInfoChange(Sender: TObject);
    procedure QueryBtnClick(Sender: TObject);
    procedure QueryWdwEdit(Sender: TObject);
    procedure QueryWdwFilter(Sender: TObject);
    procedure QueryWdwSelect(Sender: TObject);
    procedure QueryWdwOrder(Sender: TObject);
    procedure QueryWdwLoad(Sender: TObject);
    procedure QueryWdwSave(Sender: TObject);
    procedure QueryWdwPrint(Sender: TObject);
    procedure QueryWdwLookup(Sender: TObject);
    procedure QueryWdwFindNext(Sender: TObject);
    procedure QueryWdwGroup(Sender: TObject);
    procedure QueryWdwTools(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure QueryWdwDefine(Sender: TObject);
    procedure UpdateBtnClick(Sender: TObject);
    procedure UpdateInfoChange(Sender: TObject);
    procedure QueryWdwOptions(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure QueryWdwHelp(Sender: TObject);
  private
    procedure ForceBridgingJoins;
  public
  end;

var
  DemoForm: TDemoForm;

implementation

{$R *.DFM}

procedure TDemoForm.FormCreate(Sender: TObject);
begin
  LookupDlg.DataSource := QueryWdw.QuerySource;
  TablePrn.DataSource := QueryWdw.QuerySource;
end;

procedure TDemoForm.QueryBtnClick(Sender: TObject);
begin
  QueryWdw.Show;
end;

procedure TDemoForm.UpdateBtnClick(Sender: TObject);
begin
  UpdateForm.Show;
end;

procedure TDemoForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TDemoForm.QueryInfoChange(Sender: TObject);
begin
  ForceBridgingJoins;    {ensure used tables joined - see bottom procedure}
  Formatter.SQLQueryScript(QueryWdw.SQLScript);
  Formatter.UserQueryScript(QueryWdw.DisplayScript);
end;

procedure TDemoForm.UpdateInfoChange(Sender: TObject);
begin
  Formatter.SQLUpdateScript(UpdateForm.UpdateQuery.SQL);
  Formatter.UserUpdateScript(UpdateForm.ScriptMemo.Lines);
end;

procedure TDemoForm.QueryWdwEdit(Sender: TObject);
begin
  EditDlg.Execute(QueryWdw.SQLScript);
end;

procedure TDemoForm.QueryWdwFilter(Sender: TObject);
begin
  FilterDlg.Execute(fkQuery);
end;

procedure TDemoForm.QueryWdwSelect(Sender: TObject);
begin
  SelectDlg.Execute;
end;

procedure TDemoForm.QueryWdwGroup(Sender: TObject);
begin
  GroupDlg.Execute;
end;

procedure TDemoForm.QueryWdwOrder(Sender: TObject);
begin
  OrderDlg.Execute;
end;

procedure TDemoForm.QueryWdwDefine(Sender: TObject);
begin
  DefineDlg.Execute;
end;

procedure TDemoForm.QueryWdwLoad(Sender: TObject);
begin
  Archiver.Load;
end;

procedure TDemoForm.QueryWdwSave(Sender: TObject);
begin
  Archiver.Store('', QueryWdw.DisplayScript);
end;

procedure TDemoForm.QueryWdwPrint(Sender: TObject);
begin
  TablePrn.SubHeader := QueryWdw.QueryInfo.QueryTitle;
  TablePrn.Execute;
end;

procedure TDemoForm.QueryWdwLookup(Sender: TObject);
begin
  with LookupDlg do
    if Execute then LookupFirst;
end;

procedure TDemoForm.QueryWdwFindNext(Sender: TObject);
begin
  LookupDlg.LookupNext;
end;

procedure TDemoForm.QueryWdwTools(Sender: TObject);
begin
  FilterBldr.Execute;  {just to demo an alternative to FilterDlg}
end;

procedure TDemoForm.QueryWdwOptions(Sender: TObject);
begin
  OptionsForm.ShowModal;
end;

procedure TDemoForm.QueryWdwHelp(Sender: TObject);
begin
  with Application do begin
    HelpFile := ExtractFilePath(ExeName) + 'qi094.hlp';
    HelpCommand(HELP_CONTENTS, 0);
  end;
end;

{
  join links:      Vendors - Parts - Items - Orders - Customers
                                                \ - Employees
}
procedure TDemoForm.ForceBridgingJoins;
var
  Item, Part, Vendor, Order, Cust, Empl: TTableItem;
begin
  with QueryInfo do begin   {ensure bridging joins made where required -}
    Item   := Tables.ByTableRef(Items);
    Part   := Tables.ByTableRef(Parts);       {TQueryInfo gives access to the}
    Vendor := Tables.ByTableRef(Vendors);     {Tables, Fields & Joins lists}
    Order  := Tables.ByTableRef(Orders);      {of its connected FieldInfo}
    Cust   := Tables.ByTableRef(Customers);
    Empl   := Tables.ByTableRef(Employees);
    Part.Master :=                                   {A table item with its    }
      (TableUsed(Vendor) and TableUsed(Item))  or    {master property set will }
      (TableUsed(Vendor) and TableUsed(Order)) or    {always be included in    }
      (TableUsed(Vendor) and TableUsed(Cust))  or    {the FROM clause and its  }
      (TableUsed(Vendor) and TableUsed(Empl));       {common joins defined by  }
    Item.Master :=                                   {QIFormatter in the script}
      (TableUsed(Part)   and TableUsed(Order)) or
      (TableUsed(Part)   and TableUsed(Cust))  or    
      (TableUsed(Part)   and TableUsed(Empl))  or    
      (TableUsed(Vendor) and TableUsed(Order)) or    
      (TableUsed(Vendor) and TableUsed(Cust))  or    
      (TableUsed(Vendor) and TableUsed(Empl));      
    Order.Master :=
      (TableUsed(Cust) and TableUsed(Item))   or
      (TableUsed(Cust) and TableUsed(Part))   or
      (TableUsed(Cust) and TableUsed(Vendor)) or
      (TableUsed(Cust) and TableUsed(Empl))   or
      (TableUsed(Empl) and TableUsed(Item))   or
      (TableUsed(Empl) and TableUsed(Part))   or
      (TableUsed(Empl) and TableUsed(Vendor));             {phew!}
  end;
end;

end.
